'''
    Bakalářská práce
    Scrolling Song Book - zpěvník s automatickým posunem textu
    
    Hana Kalivodová
    4. 5. 2022
    
    Nahrávání zvukového signálu
'''

import pyaudio
import threading
import atexit
import numpy

# soubor s konstantami
import const

'''
    Třída zajišťující nahrávání zvukového signálu se zadanou vzorkovací frekvencí
    po jednotlivých oknech zadané délky.
    
    @author     Hana Kalivodová
'''
class MicrophoneRecorder(object):

    '''
        Inicializace počátečních hodnot
    '''
    def __init__(self):
        self.audio = pyaudio.PyAudio()
        self.stream = self.audio.open(format=pyaudio.paInt16, channels=1, start = False, rate=const.SAMPLE_RATE,
                                  input=True, frames_per_buffer=const.SAMPLES, stream_callback=self.new_frames)
        self.lock = threading.Lock()
        self.end = False
        self.frames = []
        atexit.register(self.close)

    '''
        Po nahrání jednoho rámce je volána tato funkce, která uloží data o zvukovém 
        signálu v rámci do proměnné self.frames.
        
        @param data             data zaznamenaného zvukového signálu (rámce)
        @param frame_count      počet vzorků rámce
        @param time_info        časové údaje o začátku a konci zaznamenávání
        @param status           příznak, zda je k nahrání více rámců
        
        @return None, pyaudio.paComplete    zaznamenávání zvuku bylo ukončeno
        @return None, pyaudio.paContinue    zaznamenávání zvuku pokračuje
    '''
    def new_frames(self, data, frame_count, time_info, status):
        data = numpy.fromstring(data, numpy.float16)
        
        with self.lock:
            self.frames.append(data)
            if self.end:
                return None, pyaudio.paComplete
        return None, pyaudio.paContinue
    
    '''
        Funkce vrací uložená data o nahraných rámcích a inicializuje proměnnou self.frames.
        
        @return frames      zaznamenané rámce
    '''
    def get_frames(self):
        with self.lock:
            frames = self.frames
            self.frames = []
            return frames
    
    '''
        Funkce pro spuštění nahrávání.
    '''
    def start(self):
        self.stream.start_stream()

    '''
        Funkce pro zastavení nahrávání.
    '''
    def stop(self):
        self.stream.stop_stream()
    
    '''
        Funkce pro kompletní ukončení nahrávání.
    '''
    def close(self):
        with self.lock:
            self.end = True
        self.stream.close()
        self.audio.terminate()
        
    '''
        Funkce zjišťující, zda se momentálně nahrává, či nikoli.
        
        @return True    právě probíhá záznam zvuku
        @return False   záznam zvuku neprobíhá
    '''    
    def isRunning(self):
        return self.stream.is_active()

